@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: client.c,v 1.2 2000/04/04 15:09:59 maxk Exp $
+ * $Id: client.c,v 1.3 2000/05/03 16:16:42 maxk Exp $
  */ 
 
 #include "config.h"
@@ -59,7 +59,7 @@ void client(struct vtun_host *host)
      struct sigaction sa;
      int s,opt;	
 
-     syslog(LOG_INFO,"VTun client ver %s started",VER);
+     syslog(LOG_INFO,"VTun client ver %s started",VTUN_VER);
 
      memset(&sa,0,sizeof(sa));     
      sa.sa_handler=SIG_IGN;
