@@ -17,16 +17,13 @@
  */
 
 /*
- * $Id: vtun.h,v 1.5 2000/04/17 15:23:07 maxk Exp $
+ * $Id: vtun.h,v 1.6 2000/05/03 16:16:42 maxk Exp $
  */ 
 #ifndef _VTUN_H
 #define _VTUN_H
 
 #include "llist.h"
 
-/* Version is substituted by 'make_release' */
-#define VER "2.1 04.17.2000"
-
 /* Default VTUN port */
 #define VTUN_PORT 5000
 
